@@ -12,11 +12,11 @@ from ..abstract_moc import AbstractMOC
 from .. import mocpy
 from .. import utils
 
-__author__ = "Matthieu Baumann"
+__author__ = "Matthieu Baumann, François-Xavier Pineau"
 __copyright__ = "CDS, Centre de Données astronomiques de Strasbourg"
 
 __license__ = "BSD 3-Clause License"
-__email__ = "matthieu.baumann@astro.unistra.fr"
+__email__ = "baumannmatthieu0@gmail.com, francois-xavier.pineau@astro.unistra.fr"
 
 class TimeMOC(AbstractMOC):
     """Multi-order time coverage class. Experimental"""
@@ -99,6 +99,7 @@ class TimeMOC(AbstractMOC):
         moc : `~mocpy.tmoc.TimeMOC`
             The degraded MOC.
         """
+        assert self._interval_set._intervals.shape[1] == 2
         intervals = mocpy.time_coverage_degrade(self._interval_set._intervals, new_order)
         return TimeMOC(IntervalSet(intervals, make_consistent=False), make_consistent=False)
 
@@ -125,6 +126,7 @@ class TimeMOC(AbstractMOC):
 
         times = times.reshape((times.shape[0], 1))
         intervals = np.hstack((times, times + np.uint64(1)))
+        assert intervals.shape[1] == 2
 
         # degrade the TimeMoc to the order computed from ``delta_t``
         depth = TimeMOC.time_resolution_to_order(delta_t)
@@ -282,7 +284,7 @@ class TimeMOC(AbstractMOC):
         result = self_degradation, another_moc_degradation
         return result
 
-    def intersection(self, another_moc, delta_t=DEFAULT_OBSERVATION_TIME):
+    def intersection_with_timeresolution(self, another_moc, delta_t=DEFAULT_OBSERVATION_TIME):
         """
         Intersection between self and moc. ``delta_t`` gives the possibility to the user
         to set a time resolution for performing the tmoc intersection
@@ -308,7 +310,7 @@ class TimeMOC(AbstractMOC):
         self_degraded, moc_degraded = self._process_degradation(another_moc, order_op)
         return super(TimeMOC, self_degraded).intersection(moc_degraded)
 
-    def union(self, another_moc, delta_t=DEFAULT_OBSERVATION_TIME):
+    def union_with_timeresolution(self, another_moc, delta_t=DEFAULT_OBSERVATION_TIME):
         """
         Union between self and moc. ``delta_t`` gives the possibility to the user
         to set a time resolution for performing the tmoc union
@@ -334,7 +336,7 @@ class TimeMOC(AbstractMOC):
         self_degraded, moc_degraded = self._process_degradation(another_moc, order_op)
         return super(TimeMOC, self_degraded).union(moc_degraded)
 
-    def difference(self, another_moc, delta_t=DEFAULT_OBSERVATION_TIME):
+    def difference_with_timeresolution(self, another_moc, delta_t=DEFAULT_OBSERVATION_TIME):
         """
         Difference between self and moc. ``delta_t`` gives the possibility to the user
         to set a time resolution for performing the tmoc diff
@@ -450,8 +452,7 @@ class TimeMOC(AbstractMOC):
 
         """
 
-        # max_time = Time(self._interval_set.max / TimeMOC.DAY_MICRO_SEC, format='jd', scale='tdb')
-        max_time = utils.microseconds_to_times(self._interval_set.min)
+        max_time = utils.microseconds_to_times(self._interval_set.max)
         return max_time
 
     def contains(self, times, keep_inside=True, delta_t=DEFAULT_OBSERVATION_TIME):
@@ -566,11 +567,10 @@ class TimeMOC(AbstractMOC):
 
         if self._interval_set.empty():
             import warnings
-            warnings.simplefilter('default')
             warnings.warn('This time moc is empty', UserWarning)
             return
 
-        plot_order = 15
+        plot_order = 30
         if self.max_order > plot_order:
             plotted_moc = self.degrade_to_order(plot_order)
         else:
@@ -579,6 +579,7 @@ class TimeMOC(AbstractMOC):
         min_jd = plotted_moc.min_time.jd if not view[0] else view[0].jd
         max_jd = plotted_moc.max_time.jd if not view[1] else view[1].jd
 
+        
         if max_jd < min_jd:
             raise ValueError("Invalid selection: max_jd = {0} must be > to min_jd = {1}".format(max_jd, min_jd))
 
@@ -597,10 +598,8 @@ class TimeMOC(AbstractMOC):
 
         y = np.zeros(size)
         for (s_time_us, e_time_us) in plotted_moc._interval_set._intervals:
-            #s_index = int((s_time_us / TimeMOC.DAY_MICRO_SEC - min_jd_time) / delta)
-            #e_index = int((e_time_us / TimeMOC.DAY_MICRO_SEC - min_jd_time) / delta)
-            s_index = int((utils.microseconds_to_times(s_time_us) - min_jd_time) / delta)
-            e_index = int((utils.microseconds_to_times(e_time_us) - min_jd_time) / delta)
+            s_index = int((utils.microseconds_to_times(s_time_us).jd - min_jd_time) / delta)
+            e_index = int((utils.microseconds_to_times(e_time_us).jd - min_jd_time) / delta)
             y[s_index:(e_index+1)] = 1.0
 
         # hack in case of full time mocs.
@@ -634,7 +633,7 @@ class TimeMOC(AbstractMOC):
 
         plt.show()
 
-    def save(self, path, format='fits'):
+    def save(self, path, format='fits', overwrite=False):
         """
         Writes the Time MOC to a file.
 
@@ -648,7 +647,17 @@ class TimeMOC(AbstractMOC):
             The format in which the MOC will be serialized before being saved.
             Possible formats are "fits", "ascii" or "json".
             By default, ``format`` is set to "fits".
+        overwrite : bool, optional
+            If the file already exists and you want to overwrite it, then set the  ``overwrite`` keyword.
+            Default to False.
         """
+        import os
+        file_exists = os.path.isfile(path)
+        
+        if file_exists and not overwrite:
+            raise OSError('File {} already exists! Set ``overwrite`` to '
+                          'True if you want to replace it.'.format(path))        
+        
         if format == 'fits':
             mocpy.time_moc_to_fits_file(self.max_order, self._interval_set._intervals, path)
         elif format == 'ascii':
@@ -716,6 +725,8 @@ class TimeMOC(AbstractMOC):
 
         Format can be 'ascii' or 'json', though the json format is not officially supported by the IVOA.
 
+        WARNING: the serialization must be strict, i.e. **must not** contain overlapping elements
+
         Parameters
         ----------
         format : str, optional
